@@ -1,4 +1,4 @@
-################################################################################ 
+################################################################################
 #' Scores for FLUXNET reference data in CSV format
 #' @description This function compares model output against
 #' FLUXNET measurements in CSV format. The performance of a model is
@@ -37,6 +37,16 @@
 #' @param plot.height Number that gives the plot height, e.g. 4
 #' @param numCores An integer that defines the number of cores, e.g. 2
 #' @param outputDir A string that gives the output directory, e.g. '/home/project/study'. The output will only be written if the user specifies an output directory.
+#' @param variable.name A string with the variable name, e.g. 'GPP'. If FALSE, the variable name stored in the NetCDF file will be used instead. Default is FALSE.
+#' @param numberOfMonths An integer that gives the mimimum number of months that each site should have, e.g. 60.
+#' All sites with fewer months will be excluded.
+#' @param phaseMinMax A string (either 'phaseMax' or 'phaseMin') that determines
+#' whether to assess the seasonal peak as a maximum or a minimum. The latter may be appropriate for variables
+#' that tend to be negative, such as net longwave radiation or net ecosystem exchange.
+#' @param meanPerGridCell Logical. If TRUE, then values from different sites that
+#' are located in the same grid cell are averaged. Default is set to TRUE.
+#' @param myCex A number that determines the size of the dots in the Figure.
+#' @param subcaption A string that defines the subcaption of the figure, e.g. '(a)'.
 #' @return (1) Figures in PDF format that show maps of
 #' the model data at the location of FLUXNET sites
 #' (mean, \eqn{mod.mean}; interannual-variability, \eqn{mod.iav}; month of annual cycle maximum, \eqn{mod.max.month}),
@@ -49,16 +59,26 @@
 #'
 #' (2) Four text files: (i) score values and (ii) score inputs for each individual
 #' site, and (iii) score values and (iv) score inputs averaged across sites.
-#' when averaging over all station.
 #'
 #' @examples
 #' \donttest{
 #' library(amber)
-#' library(parallel)
+#' library(classInt)
 #' library(doParallel)
 #' library(foreach)
+#' library(Hmisc)
+#' library(latex2exp)
 #' library(ncdf4)
+#' library(parallel)
 #' library(raster)
+#' library(rgdal)
+#' library(rgeos)
+#' library(scico)
+#' library(sp)
+#' library(stats)
+#' library(utils)
+#' library(viridis)
+#' library(xtable)
 #'
 #' # (1) Global plots on a regular grid
 #' long.name <- 'Gross primary productivity'
@@ -74,27 +94,10 @@
 #' scores.fluxnet.csv(long.name, nc.mod, ref.csv, mod.id, ref.id,
 #' unit.conv.mod, unit.conv.ref, variable.unit)
 #'
-#' # Additional parameters:
-#' score.weights <- c(1,2,1,1,1) # score weights of S_bias, S_rmse, S_phase, S_iav, S_dist
-#' rotate.me <- TRUE
-#' irregular <- FALSE
-#' my.projection <- '+proj=longlat +ellps=WGS84'
-#' shp.filename <- system.file('extdata/ne_110m_land/ne_110m_land.shp', package = 'amber')
-#' my.xlim <- c(-180, 180)  # longitude range that you wish to plot
-#' my.ylim <- c(-60, 85)  # latitude range that you wish to plot
-#' plot.width <- 8
-#' plot.height <- 3.8
-#' numCores <- 2
-#'
-#' scores.fluxnet.csv(long.name, nc.mod, ref.csv, mod.id, ref.id,
-#' unit.conv.mod, unit.conv.ref, variable.unit, score.weights, rotate.me,
-#' irregular, my.projection, shp.filename, my.xlim, my.ylim, plot.height,
-#' numCores = 2)
-#'
 #' # To zoom into a particular region:
 #' scores.fluxnet.csv(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.conv.mod,
-#' unit.conv.ref, variable.unit, score.weights, my.xlim = c(-150, -60), my.ylim = c(20, 60),
-#' plot.width = 6, plot.height = 3.8)
+#' unit.conv.ref, variable.unit, score.weights = c(1, 2, 1, 1, 1),
+#' my.xlim = c(-150, -60), my.ylim = c(20, 60), plot.width = 6, plot.height = 3.8)
 #'
 #' # (2) Regional plots on a rotated grid
 #' nc.mod <- system.file('extdata/modelRotated', 'gpp_monthly.nc', package = 'amber')
@@ -108,8 +111,9 @@
 #' rotate.me <- FALSE
 #' irregular <- TRUE
 #' my.projection <-'+proj=ob_tran +o_proj=longlat +o_lon_p=83. +o_lat_p=42.5 +lon_0=263.'
-#' shp.filename <- system.file('extdata/ne_50m_admin_0_countries/ne_50m_admin_0_countries.shp',
-#'  package = 'amber')
+#' # shp.filename <- system.file('extdata/ne_50m_admin_0_countries/ne_50m_admin_0_countries.shp',
+#' # package = 'amber')
+#' shp.filename <- system.file("extdata/ne_110m_land/ne_110m_land.shp", package = "amber")
 #' my.xlim <- c(-171, 0) # longitude range that you wish to plot
 #' my.ylim <- c(32, 78) # latitude range that you wish to plot
 #' plot.width <- 7
@@ -117,35 +121,36 @@
 #' numCores <- 2
 #'
 #' scores.fluxnet.csv(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.conv.mod,
-#' unit.conv.ref, variable.unit, score.weights, rotate.me, irregular,
-#' my.projection,
+#' unit.conv.ref, variable.unit, score.weights, rotate.me, irregular, my.projection,
 #' shp.filename, my.xlim, my.ylim)
-#' }
+#' } #donttest
+#'
 #' @export
-scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.conv.mod, unit.conv.ref, variable.unit, 
-    score.weights = c(1, 2, 1, 1, 1), rotate.me = TRUE, irregular = FALSE, my.projection = "+proj=longlat +ellps=WGS84", 
-    shp.filename = system.file("extdata/ne_110m_land/ne_110m_land.shp", package = "amber"), my.xlim = c(-180, 180), my.ylim = c(-60, 
-        85), plot.width = 8, plot.height = 3.8, numCores = 2, outputDir = FALSE) {
-    
+scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.conv.mod, unit.conv.ref, variable.unit, score.weights = c(1,
+    2, 1, 1, 1), rotate.me = TRUE, irregular = FALSE, my.projection = "+proj=longlat +ellps=WGS84", shp.filename = system.file("extdata/ne_110m_land/ne_110m_land.shp",
+    package = "amber"), my.xlim = c(-180, 180), my.ylim = c(-60, 85), plot.width = 8, plot.height = 3.8, numCores = 2, outputDir = FALSE,
+    variable.name = FALSE, numberOfMonths = 36, phaseMinMax = "phaseMax", meanPerGridCell = TRUE, myCex = 0.5, subcaption = "") {
+
     #---------------------------------------------------------------------------
-    
+
     # (I) Data preparation
-    
+
     #---------------------------------------------------------------------------
-    
+
     # (1) Reproject data from an irregular to a regular grid if 'irregular=TRUE'
-    
+
     if (irregular == TRUE) {
         regular <- "+proj=longlat +ellps=WGS84"
         rotated <- my.projection
         # get variable name
-        nc <- ncdf4::nc_open(nc.mod)
-        variable.name <- base::names(nc[["var"]])
-        variable.name <- variable.name[1]
-        variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
-        variable.name <- toupper(variable.name)  # make variable name upper-case
-        ncdf4::nc_close(nc)
-        
+        if (variable.name == FALSE) {
+            nc <- ncdf4::nc_open(nc.mod)
+            variable.name <- base::names(nc[["var"]])
+            variable.name <- variable.name[1]
+            variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
+            variable.name <- toupper(variable.name)  # make variable name upper-case
+            ncdf4::nc_close(nc)
+        }
         # process model data
         my.nc <- nc.mod
         nc <- ncdf4::nc_open(my.nc)
@@ -153,7 +158,7 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         lon <- ncdf4::ncvar_get(nc, "lon")
         lat <- ncdf4::ncvar_get(nc, "lat")
         time <- ncdf4::ncvar_get(nc, "time")
-        # 
+        #
         nCol <- base::length(lon[, 1])
         nRow <- base::length(lon[1, ])
         nTime <- base::length(time)
@@ -167,14 +172,14 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         end.date <- max(dates)
         start.date <- format(as.Date(start.date), "%Y-%m")
         end.date <- format(as.Date(end.date), "%Y-%m")
-        
+
         lon <- base::matrix(lon, ncol = 1)
         lat <- base::matrix(lat, ncol = 1)
-        
+
         lonLat <- base::data.frame(lon, lat)
         sp::coordinates(lonLat) <- ~lon + lat
         raster::projection(lonLat) <- regular
-        lonLat <- sp::spTransform(lonLat, sp::CRS(rotated))
+        suppressWarnings(lonLat <- sp::spTransform(lonLat, sp::CRS(rotated)))
         myExtent <- raster::extent(lonLat)
         # create an empty raster
         r <- raster::raster(ncols = nCol, nrows = nRow)  # empty raster
@@ -195,22 +200,32 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         mod <- myStack
         start.date.mod <- start.date
         end.date.mod <- end.date
-        
+
         #-----------------------------------------------------------------------
-        
+
         # Process reference data
-        
+
         #-----------------------------------------------------------------------
-        
+
         # The raw data has the row names lat, lon, time, data, siteID, IGBPcode.
-        
+
         # The desired output is a data frame with the column names:
-        
+
         # lon, lat, YYYY-01, YYYY-02, (...),
-        
+
         # where each row presents one site
-        
+
         nc <- utils::read.csv(ref.csv)
+
+        # exclude stations that don't have enough data
+        nc[nc == -9999] <- NA
+        nMonths <- stats::na.omit(nc)
+        nMonths <- tapply(nMonths[[4]], nMonths$sitename, length)
+        sitesWithSufficientData <- nMonths[nMonths >= numberOfMonths]
+        sitesWithSufficientData <- names(sitesWithSufficientData)
+        nc <- subset(nc, sitename %in% c(sitesWithSufficientData))
+        nc <- droplevels(nc)
+
         # get time range
         dates.ref <- nc$time
         dates.ref <- as.Date(dates.ref)
@@ -218,11 +233,11 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         dates.ref <- unique(dates.ref)
         start.date.ref <- min(dates.ref)
         end.date.ref <- max(dates.ref)
-        
+
         site.id <- unique(nc$sitename)
         nSites <- length(site.id)
         allSites <- split(nc, nc$sitename)
-        
+
         # Restructure data
         cl <- parallel::makePSOCKcluster(numCores)
         doParallel::registerDoParallel(cl)
@@ -232,7 +247,6 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
             date <- as.Date(date)
             date <- format(as.Date(date), "%Y-%m")  # only year and month
             values <- t(singleSite[4])
-            values[values == -9999] <- NA
             ID <- as.character(site.id[i])
             lat <- singleSite[1, 1]
             lon <- singleSite[1, 2]
@@ -246,23 +260,23 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
             myMerge <- merge(x, y, all.x = TRUE, all.y = TRUE)
             return(myMerge)
         }
-        
+
         # merge all individual sites and sort columns by date
         ref <- Reduce(fun.merge, eachSite)
         siteID <- as.character(ref$siteID)
         lon <- ref$lon
         lat <- ref$lat
-        
+
         # reproject coordinates to rotated grid
         lon <- base::matrix(lon, ncol = 1)
         lat <- base::matrix(lat, ncol = 1)
         lonLat <- base::data.frame(lon, lat)
         sp::coordinates(lonLat) <- ~lon + lat
         raster::projection(lonLat) <- regular
-        lonLat <- sp::spTransform(lonLat, sp::CRS(rotated))
+        suppressWarnings(lonLat <- sp::spTransform(lonLat, sp::CRS(rotated)))
         lon <- sp::coordinates(lonLat)[, 1]
         lat <- sp::coordinates(lonLat)[, 2]
-        
+
         data <- ref[4:ncol(ref)]  # data only
         data <- data[, order(names(data))]
         ref <- data.frame(siteID, lon, lat, data * unit.conv.ref)
@@ -271,50 +285,63 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         ref <- ref[, -1]  # drop siteID column
         rownames(ref) <- myRownames
     } else {
-        
+
         #-----------------------------------------------------------------------
-        
+
         # (2) Process data if 'irregular=FALSE'
-        
+
         #-----------------------------------------------------------------------
-        
+
         # model data
-        nc <- ncdf4::nc_open(nc.mod)
-        variable.name <- names(nc[["var"]])
-        ncdf4::nc_close(nc)
-        variable.name <- variable.name[length(variable.name)]  # take the last variable (relevant for CanESM5)
-        variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
-        variable.name <- toupper(variable.name)  # make variable name upper-case
+        if (variable.name == FALSE) {
+            nc <- ncdf4::nc_open(nc.mod)
+            variable.name <- names(nc[["var"]])
+            ncdf4::nc_close(nc)
+            variable.name <- variable.name[length(variable.name)]  # take the last variable (relevant for CanESM5)
+            variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
+            variable.name <- toupper(variable.name)  # make variable name upper-case
+        }
         mod <- raster::brick(nc.mod)
         dates.mod <- raster::getZ(mod)
         dates.mod <- format(as.Date(dates.mod), "%Y-%m")  # only year and month
         start.date.mod <- min(dates.mod)
         end.date.mod <- max(dates.mod)
         myExtent <- c(-180, 180, -90, 90)  # this can be done in a smarter way
-        
+
         # Process reference data
-        
+
         # The raw data has the row names lat, lon, time, data, siteID, IGBPcode.
-        
+
         # The desired output is a data frame with the column names:
-        
+
         # lon, lat, YYYY-01, YYYY-02, (...),
-        
+
         # where each row presents one site
-        
+
         nc <- utils::read.csv(ref.csv)
+
+        # exclude stations that don't have enough data
+        nc[nc == -9999] <- NA
+        nMonths <- stats::na.omit(nc)
+        nMonths <- tapply(nMonths[[4]], nMonths$sitename, length)
+        sitesWithSufficientData <- nMonths[nMonths >= numberOfMonths]
+        sitesWithSufficientData <- names(sitesWithSufficientData)
+        nc <- subset(nc, sitename %in% c(sitesWithSufficientData))
+        nc <- droplevels(nc)
+
         # get time range
         dates.ref <- nc$time
         dates.ref <- as.Date(dates.ref)
         dates.ref <- format(as.Date(dates.ref), "%Y-%m")  # only year and month
         dates.ref <- unique(dates.ref)
+        dates.ref <- sort(dates.ref)  # Added on May 4, 2020
         start.date.ref <- min(dates.ref)
         end.date.ref <- max(dates.ref)
-        
+
         site.id <- unique(nc$sitename)
         nSites <- length(site.id)
         allSites <- split(nc, nc$sitename)
-        
+
         # Restructure data
         cl <- parallel::makePSOCKcluster(numCores)
         doParallel::registerDoParallel(cl)
@@ -324,7 +351,6 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
             date <- as.Date(date)
             date <- format(as.Date(date), "%Y-%m")  # only year and month
             values <- t(singleSite[4])
-            values[values == -9999] <- NA
             ID <- as.character(site.id[i])
             lat <- singleSite[1, 1]
             lon <- singleSite[1, 2]
@@ -338,8 +364,10 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
             myMerge <- merge(x, y, all.x = TRUE, all.y = TRUE)
             return(myMerge)
         }
-        
-        # merge all individual sites and sort columns by date
+
+        # merge all individual sites and sort columns by date the resulting list has one element with a structure like this:
+
+        # siteID lon lat 2005-01 2005-02 2005-03 (...)  1 aaa -10 -20 1 NA 2 2 bbb -30 -15 4 5 9 (...)
         ref <- Reduce(fun.merge, eachSite)
         siteID <- as.character(ref$siteID)
         lon <- ref$lon
@@ -352,14 +380,14 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         ref <- ref[, -1]  # drop siteID column
         rownames(ref) <- myRownames
     }
-    
+
     #---------------------------------------------------------------------------
-    
+
     # find common time period
     start.date <- max(start.date.mod, start.date.ref)
     end.date <- min(end.date.mod, end.date.ref)
     # subset common time period for model data
-    mod <- mod[[which(format(as.Date(raster::getZ(mod)), "%Y-%m") >= start.date & format(as.Date(raster::getZ(mod)), "%Y-%m") <= 
+    mod <- mod[[which(format(as.Date(raster::getZ(mod)), "%Y-%m") >= start.date & format(as.Date(raster::getZ(mod)), "%Y-%m") <=
         end.date)]]
     # subset common time period for reference data get a sequence of common dates
     dates <- seq(from = as.Date(paste(start.date, "15", sep = "-")), to = as.Date(paste(end.date, "15", sep = "-")), by = "month")
@@ -371,25 +399,61 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     dates.ref <- data.frame(index, dates.ref)
     colnames(dates.ref) <- c("index", "dates")
     ## merge common dates and reference dates
-    
+
     # If the time covered by the reference data exceeds the time covered by the
-    
+
     # model data, then the reference data set will be subset using the index
-    
+
     column.id <- merge(dates.ref, common.dates, by = "dates")
     column.id <- column.id$index
     ref <- data.frame(ref[2 + column.id])  # 2+ because column 1 and 2 are are lon and lat
     colnames(ref) <- c(dates)
     ref <- intFun.site.points(lon, lat, ref)
-    
-    # compute statistics for each site
-    
-    # extract model data for each site
+
     if (rotate.me == TRUE) {
-        mod <- raster::rotate(mod)
+        suppressWarnings(mod <- raster::rotate(mod))
     }
+
+    # compute mean value per grid cell
+
+    if (meanPerGridCell == TRUE) {
+
+        # get cell number and coordinates for each grid cell
+        myGrid <- raster::subset(mod, 1:1)
+        myGrid <- raster::setValues(myGrid, 1)  # this ensures that all grid cells are considered, including ocean grid cells
+        modLonLat <- raster::rasterToPoints(myGrid, spatial = TRUE)
+        modcn <- raster::extract(myGrid, modLonLat, cellnumbers = TRUE)
+        lonLatCN <- data.frame(sp::coordinates(modLonLat), modcn[, 1])
+        colnames(lonLatCN) <- c("lon", "lat", "Group.1")
+
+        # get cell number for each site
+        cn <- raster::extract(mod, ref, cellnumbers = TRUE)
+        cn <- data.frame(cn[, 1])
+        colnames(cn) <- "Group.1"
+
+        # add cell numbers to reference data
+        ref.cn <- data.frame(cn, ref)
+
+        # compute mean value per grid cell (mpg)
+        index <- list(ref.cn$Group.1)
+        mpg <- stats::aggregate(as.data.frame(ref), by = index, FUN = mean, na.rm = TRUE)
+        mpg <- subset(mpg, select = -c(lon, lat))
+
+        # add the grid cell coordinates that correspond to each site
+        mpg <- merge(mpg, lonLatCN, by = "Group.1")
+        lon <- mpg$lon
+        lat <- mpg$lat
+
+        mpg <- subset(mpg, select = -c(Group.1, lon, lat))
+        ref <- intFun.site.points(lon, lat, mpg)
+    }
+
+    # compute statistics for each site
+
+    # extract model data for each site
+
     mod <- mod * unit.conv.mod
-    mod <- raster::extract(mod, ref, method = "bilinear")
+    mod <- raster::extract(mod, ref, method = "simple")
     mod <- data.frame(mod)
     # rownames(mod) <- site.id
     ref <- as.data.frame(ref)
@@ -402,17 +466,17 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     meta.data.ref <- paste(variable.name, ref.id, "from", start.date, "to", end.date, sep = "_")
     meta.data.com <- paste(variable.name, mod.id, "vs", ref.id, "from", start.date, "to", end.date, sep = "_")
     #---------------------------------------------------------------------------
-    
+
     # (II) Statistical analysis
-    
+
     #---------------------------------------------------------------------------
-    
+
     # The time period of the reference data varies among sites.
-    
+
     # Some sites are located in the model ocean.
-    
+
     # Exclude data that do not have a corresponding data set in mod or ref.
-    
+
     mask.mod <- (mod - mod + 1)
     mask.ref <- (ref - ref + 1)
     mask <- mask.mod * mask.ref
@@ -425,18 +489,24 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     # transpose data, where rows are dates and columns are sites this is necessary for using mapply below
     mod <- data.frame(t(mod))
     ref <- data.frame(t(ref))
-    
     #---------------------------------------------------------------------------
-    
+
     # (1) bias
-    
+
     #---------------------------------------------------------------------------
     mod.mean <- apply(mod, 2, mean, na.rm = TRUE)  # time mean
     ref.mean <- apply(ref, 2, mean, na.rm = TRUE)  # time mean
     weights <- ref.mean  # weights used for spatial integral
     bias <- mod.mean - ref.mean  # time mean
     ref.sd <- apply(ref, 2, sd, na.rm = TRUE)  # standard deviation of reference data
-    epsilon_bias <- abs(bias)/ref.sd  # relative error
+    epsilon_bias <- abs(bias)/ref.sd
+    epsilon_bias[epsilon_bias == Inf] <- NA  # relative error
+    # If this function is used for in-situ measurments of LAI, compute relative bias as follows:
+    if (variable.name == "LAI")
+        {
+            epsilon_bias <- abs(bias)/abs(ref.mean)
+            epsilon_bias[epsilon_bias == Inf] <- NA
+        }  # relative error
     bias.score <- exp(-epsilon_bias)  # bias score as a function of space
     S_bias_not.weighted <- mean(bias.score, na.rm = TRUE)  # scalar score (not weighted)
     # calculate the weighted scalar score
@@ -447,19 +517,21 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     mod.mean.scalar <- mean(mod.mean, na.rm = TRUE)  # global mean value
     ref.mean.scalar <- mean(ref.mean, na.rm = TRUE)  # global mean value
     bias.scalar <- mean(bias, na.rm = TRUE)  # global mean value
+    bias.scalar.rel <- (mod.mean.scalar - ref.mean.scalar)/abs(ref.mean.scalar) * 100
     ref.sd.scalar <- mean(ref.sd, na.rm = TRUE)
-    epsilon_bias.scalar <- mean(epsilon_bias, na.rm = TRUE)
-    
+    epsilon_bias.scalar <- stats::median(epsilon_bias, na.rm = TRUE)
+
     #---------------------------------------------------------------------------
-    
+
     # (2) root mean square error (rmse)
-    
+
     #---------------------------------------------------------------------------
     rmse <- mapply(intFun.rmse, mod, ref)  # compute rmse
     mod.anom <- data.frame(apply(mod, 2, intFun.anom))  # compute anomalies
     ref.anom <- data.frame(apply(ref, 2, intFun.anom))  # compute anomalies
     crmse <- mapply(intFun.crmse, mod.anom, ref.anom)
-    epsilon_rmse <- crmse/ref.sd  # relative error
+    epsilon_rmse <- crmse/ref.sd
+    epsilon_rmse[epsilon_rmse == Inf] <- NA  # relative error
     rmse.score <- exp(-epsilon_rmse)  # rmse score as a function of space
     S_rmse_not.weighted <- mean(rmse.score, na.rm = TRUE)  # scalar score (not weighted)
     # calculate the weighted scalar score
@@ -469,7 +541,7 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     # compute global mean values of score input(s)
     rmse.scalar <- mean(rmse, na.rm = TRUE)  # global mean value
     crmse.scalar <- mean(crmse, na.rm = TRUE)  # global mean value
-    epsilon_rmse.scalar <- mean(epsilon_rmse, na.rm = TRUE)
+    epsilon_rmse.scalar <- stats::median(epsilon_rmse, na.rm = TRUE)
     # (3) phase shift make monthly means month index is the same for mod and ref, given the temporal subset made above
     month <- format(as.Date(paste(dates, "15", sep = "-")), format = "%m")
     month <- as.numeric(month)
@@ -488,9 +560,22 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     # drop 'month' column
     mod.clim.mly <- subset(mod.clim.mly, select = -c(month))
     ref.clim.mly <- subset(ref.clim.mly, select = -c(month))
-    # find month with max value
-    mod.max.month <- apply(mod.clim.mly, 2, which.max)
-    ref.max.month <- apply(ref.clim.mly, 2, which.max)
+
+    # find month of seasonal peak
+
+    # In most cases, we are interested in the timing of the seasonal maximum value
+
+    # In some cases, however, the seasonal peak is a minimum, e.g. NEE = RECO - GPP
+    if (phaseMinMax == "phaseMax") {
+        mod.max.month <- apply(mod.clim.mly, 2, which.max)
+        ref.max.month <- apply(ref.clim.mly, 2, which.max)
+    }
+
+    if (phaseMinMax == "phaseMin") {
+        mod.max.month <- apply(mod.clim.mly, 2, which.min)
+        ref.max.month <- apply(ref.clim.mly, 2, which.min)
+    }
+
     mod.max.month <- as.numeric(mod.max.month)
     mod.max.month <- data.frame(as.numeric(mod.max.month))
     ref.max.month <- data.frame(as.numeric(ref.max.month))
@@ -507,25 +592,25 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     phase.scalar <- mean(phase, na.rm = TRUE)  # global mean value
     mod.max.month.scalar <- mean(mod.max.month[, 1], na.rm = TRUE)  # global mean value
     ref.max.month.scalar <- mean(ref.max.month[, 1], na.rm = TRUE)  # global mean value
-    
+
     #---------------------------------------------------------------------------
-    
+
     # (4) interannual variability
-    
+
     #---------------------------------------------------------------------------
-    
-    # This approach assumes that all data start in Jan all months after the last Dec will be dropped if data does not end in
-    # Dec
+
+    # This approach assumes that all data start in Jan all months after the last Dec will be dropped if data does not end in Dec
     mod.anom <- intFun.anom.mly(mod, mod.clim.mly.mm)
     ref.anom <- intFun.anom.mly(ref, ref.clim.mly.mm)
     mod.iav <- apply(mod.anom, 2, intFun.iav)
     ref.iav <- apply(ref.anom, 2, intFun.iav)
-    
+
     # set values close to zero to NA
     ref.iav.na <- ref.iav
     ref.iav.na[ref.iav.na < 10^(-5)] <- NA
-    
-    epsilon_iav <- abs((mod.iav - ref.iav))/ref.iav.na  # I changed Eq. 26 so that epsilon_iav > =  0
+
+    epsilon_iav <- abs((mod.iav - ref.iav))/ref.iav.na
+    epsilon_iav[epsilon_iav == Inf] <- NA  # I changed Eq. 26 so that epsilon_iav > =  0
     iav.score <- exp(-epsilon_iav)  # iav score as a function of space
     S_iav_not.weighted <- mean(iav.score, na.rm = TRUE)  # scalar score (not weighted)
     # calculate the weighted scalar score
@@ -535,14 +620,14 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     # compute global mean values of score input(s)
     mod.iav.scalar <- mean(mod.iav, na.rm = TRUE)  # global mean value
     ref.iav.scalar <- mean(ref.iav, na.rm = TRUE)  # global mean value
-    epsilon_iav.scalar <- mean(epsilon_iav, na.rm = TRUE)  # global mean value
-    
+    epsilon_iav.scalar <- stats::median(epsilon_iav, na.rm = TRUE)  # global mean value
+
     #---------------------------------------------------------------------------
-    
+
     # (5) dist
-    
+
     #---------------------------------------------------------------------------
-    
+
     mod.sigma.scalar <- sd(mod.mean, na.rm = TRUE)  # standard deviation of period mean data
     ref.sigma.scalar <- sd(ref.mean, na.rm = TRUE)  # standard deviation of period mean data
     sigma <- mod.sigma.scalar/ref.sigma.scalar
@@ -551,13 +636,13 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     reg <- stats::lm(y ~ x)
     R <- sqrt(summary(reg)$r.squared)
     S_dist <- 2 * (1 + R)/(sigma + 1/sigma)^2  # weighting does not apply
-    
+
     #---------------------------------------------------------------------------
-    
+
     # Scores
-    
+
     #---------------------------------------------------------------------------
-    
+
     w.bias <- score.weights[1]
     w.rmse <- score.weights[2]
     w.phase <- score.weights[3]
@@ -569,7 +654,7 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     S_phase <- S_phase_not.weighted
     S_iav <- S_iav_not.weighted
     # weight importance of statisitcal metrics and compute overall score
-    S_overall <- (w.bias * S_bias + w.rmse * S_rmse + w.phase * S_phase + w.iav * S_iav + w.dist * S_dist)/(w.bias + w.rmse + 
+    S_overall <- (w.bias * S_bias + w.rmse * S_rmse + w.phase * S_phase + w.iav * S_iav + w.dist * S_dist)/(w.bias + w.rmse +
         w.phase + w.iav + w.dist)
     scores <- data.frame(variable.name, ref.id, S_bias, S_rmse, S_phase, S_iav, S_dist, S_overall)
     scores_not.weighted <- scores
@@ -578,11 +663,49 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     S_rmse <- S_rmse_weighted
     S_phase <- S_phase_weighted
     S_iav <- S_iav_weighted
-    S_overall <- (w.bias * S_bias + w.rmse * S_rmse + w.phase * S_phase + w.iav * S_iav + w.dist * S_dist)/(w.bias + w.rmse + 
+    S_overall <- (w.bias * S_bias + w.rmse * S_rmse + w.phase * S_phase + w.iav * S_iav + w.dist * S_dist)/(w.bias + w.rmse +
         w.phase + w.iav + w.dist)
     scores <- data.frame(variable.name, ref.id, S_bias, S_rmse, S_phase, S_iav, S_dist, S_overall)
     scores_weighted <- scores
-    # 
+    #---------------------------------------------------------------------------
+
+    if (variable.name == "LAI") {
+        S_bias <- S_bias_not.weighted
+        S_rmse <- NA
+        S_phase <- NA
+        S_iav <- NA
+        # weight importance of statisitcal metrics and compute overall score
+        S_overall <- (S_bias + S_dist)/2
+        scores <- data.frame(variable.name, ref.id, S_bias, S_rmse, S_phase, S_iav, S_dist, S_overall)
+        scores_not.weighted <- scores
+        # weighted (except for S_dist)
+        S_bias <- S_bias_weighted
+        S_rmse <- NA
+        S_phase <- NA
+        S_iav <- NA
+        S_overall <- S_dist
+        S_overall <- (S_bias + S_dist)/2
+
+        scores <- data.frame(variable.name, ref.id, S_bias, S_rmse, S_phase, S_iav, S_dist, S_overall)
+        scores_weighted <- scores
+
+        ref.sd.scalar <- NA
+        rmse.scalar <- NA
+        crmse.scalar <- NA
+        ref.sd.scalar <- NA
+        epsilon_rmse.scalar <- NA
+        S_rmse_not.weighted <- NA
+        mod.max.month.scalar <- NA
+        ref.max.month.scalar <- NA
+        phase.scalar <- NA
+        S_phase_not.weighted <- NA
+        mod.iav.scalar <- NA
+        ref.iav.scalar <- NA
+        epsilon_iav.scalar <- NA
+        S_iav_not.weighted <- NA
+    }
+
+    #
     scores <- rbind(scores_not.weighted, scores_weighted)
     rownames(scores) <- c("not.weighted", "weighted")
     if (outputDir != FALSE) {
@@ -597,9 +720,9 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         utils::write.table(all.score.values, paste(outputDir, "/", "allscorevalues", "-", variable.name, "-", ref.id, sep = ""))
     }
     # selected score inputs
-    scoreinputs <- data.frame(long.name, variable.name, ref.id, variable.unit, mod.mean.scalar, ref.mean.scalar, bias.scalar, 
-        ref.sd.scalar, epsilon_bias.scalar, S_bias_not.weighted, rmse.scalar, crmse.scalar, ref.sd.scalar, epsilon_rmse.scalar, 
-        S_rmse_not.weighted, mod.max.month.scalar, ref.max.month.scalar, phase.scalar, S_phase_not.weighted, mod.iav.scalar, 
+    scoreinputs <- data.frame(long.name, variable.name, ref.id, variable.unit, mod.mean.scalar, ref.mean.scalar, bias.scalar,
+        bias.scalar.rel, ref.sd.scalar, epsilon_bias.scalar, S_bias_not.weighted, rmse.scalar, crmse.scalar, ref.sd.scalar, epsilon_rmse.scalar,
+        S_rmse_not.weighted, mod.max.month.scalar, ref.max.month.scalar, phase.scalar, S_phase_not.weighted, mod.iav.scalar,
         ref.iav.scalar, epsilon_iav.scalar, S_iav_not.weighted, mod.sigma.scalar, ref.sigma.scalar, sigma, R, S_dist)
     if (outputDir != FALSE) {
         utils::write.table(scoreinputs, paste(outputDir, "/", "scoreinputs", "_", meta.data.com, sep = ""))
@@ -616,15 +739,15 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     mmi.mean <- intFun.min.max.int.mod.ref(mod.mean, ref.mean)
     mmi.max.month <- c(1, 12, 1)
     mmi.iav <- intFun.min.max.int.mod.ref(mod.iav, ref.iav)
-    
+
     #---------------------------------------------------------------------------
-    
+
     # Add metadata:
-    
+
     # 1. figure title (e.g. Mean_nee_ModID_123_from_1982-01_to_2008-12)
-    
+
     # 2. min, max, interval used in legend (e.g. 0, 1, 0.1)
-    
+
     # 3. legend bar text (e.g. 'score (-)')
     attr(mod.mean, "metadata") <- list(paste("Mean", meta.data.mod, sep = "_"), mmi.mean, variable.unit)
     attr(ref.mean, "metadata") <- list(paste("Mean", meta.data.ref, sep = "_"), mmi.mean, variable.unit)
@@ -640,15 +763,21 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
     attr(ref.iav, "metadata") <- list(paste("IAV", meta.data.ref, sep = "_"), mmi.iav, variable.unit)
     attr(iav.score, "metadata") <- list(paste("IAV_score", meta.data.com, sep = "_"), mmi.iav.score, "score (-)")
     # write data to file
-    stat.metric <- data.frame(lon, lat, mod.mean, ref.mean, bias, bias.score, crmse, rmse.score, mod.max.month, ref.max.month, 
+    stat.metric <- data.frame(lon, lat, mod.mean, ref.mean, bias, bias.score, crmse, rmse.score, mod.max.month, ref.max.month,
         phase, phase.score, mod.iav, ref.iav, iav.score)
-    colnames(stat.metric) <- c("lon", "lat", "mod.mean", "ref.mean", "bias", "bias.score", "crmse", "rmse.score", "mod.max.month", 
+    colnames(stat.metric) <- c("lon", "lat", "mod.mean", "ref.mean", "bias", "bias.score", "crmse", "rmse.score", "mod.max.month",
         "ref.max.month", "phase", "phase.score", "mod.iav", "ref.iav", "iav.score")
-    my.filename <- paste(variable.name, "FLUXNET", sep = "_")
+    if (variable.name == "LAI") {
+        stat.metric <- data.frame(lon, lat, mod.mean, ref.mean, bias, bias.score)
+        colnames(stat.metric) <- c("lon", "lat", "mod.mean", "ref.mean", "bias", "bias.score")
+    }
+
+
+    my.filename <- paste(variable.name, ref.id, sep = "_")
     if (outputDir != FALSE) {
         utils::write.table(stat.metric, paste(outputDir, my.filename, sep = "/"))
     }
-    
+
     land <- intFun.coast(my.xlim, my.ylim, my.projection, shp.filename)  # reproject coastline
     # loop through all layers of the raster stack
     lon2 <- stat.metric$lon
@@ -677,34 +806,61 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         max <- min.max.int[2]
         interval <- min.max.int[3]
         my.breaks <- round(seq(min, max, interval), 3)  # breaks of the colors
+        my.breaks.bias <- round(seq(min - interval, max + interval, interval), 3)  # location of color breaks
         my.labels <- round(seq(min, max, interval), 3)  # locations where to set the labels
         my.col <- viridis::viridis(n = length(my.breaks) - 1, direction = -1)
         my.col.bias <- scico::scico(n = length(my.breaks) - 1, palette = "vik")
+        my.col.bias <- c("magenta", my.col.bias, "black")  # add colors for outliers
         my.col.phase <- grDevices::rainbow(n = length(my.breaks) - 1)
-        if (i == 5) 
+        if (i == 5)
             {
                 my.col <- my.col.bias
+                my.breaks <- my.breaks.bias
+                # Change values of outliers for the purpose of the legend
+                data <- data[[1]]
+                data[data > max] <- max + interval/2
+                data[data < min] <- min - interval/2
+                values <- data.frame(data)
+                colnames(values) <- "values"
+                # data <- intFun.site.points(lon, lat, values) # make spatial points
+                data <- data.frame(lon, lat, values)
+                bias.pos <- subset(data, data$values >= 0)
+                bias.neg <- subset(data, data$values < 0)
+
+                bias.pos <- intFun.site.points(bias.pos$lon, bias.pos$lat, bias.pos$values)
+                bias.neg <- intFun.site.points(bias.neg$lon, bias.neg$lat, bias.neg$values)
+
             }  # divergent color scheme for bias plots
-        if (i == 9) 
+        if (i == 9)
             {
                 my.col <- my.col.phase
             }  # circular color scheme for phase
-        if (i == 10) 
+        if (i == 10)
             {
                 my.col <- my.col.phase
             }  # circular color scheme for phase
-        if (i == 11) 
+        if (i == 11)
             {
                 my.col <- my.col.phase
             }  # circular color scheme for phase
-        
+
         my.axis.args <- list(at = my.labels, labels = my.labels, cex.axis = 1)
         my.legend.args <- list(text = legend.bar.text, side = 2, font = 1, line = 1, cex = 1)
-        
+
         colors <- cut(values$values, breaks = my.breaks, labels = my.col, include.lowest = TRUE)
         colors <- toString(colors)
         colors <- unlist(strsplit(colors, split = ", "))
-        
+
+        if (i == 5) {
+            colors.pos <- cut(bias.pos$data, breaks = my.breaks, labels = my.col, include.lowest = TRUE)
+            colors.pos <- toString(colors.pos)
+            colors.pos <- unlist(strsplit(colors.pos, split = ", "))
+
+            colors.neg <- cut(bias.neg$data, breaks = my.breaks, labels = my.col, include.lowest = TRUE)
+            colors.neg <- toString(colors.neg)
+            colors.neg <- unlist(strsplit(colors.neg, split = ", "))
+        }
+
         # plot
         oldpar <- graphics::par(mfrow = c(1, 2))
         on.exit(graphics::par(oldpar))
@@ -725,21 +881,32 @@ scores.fluxnet.csv <- function(long.name, nc.mod, ref.csv, mod.id, ref.id, unit.
         dummy <- matrix(dummy, nrow = 180)
         dummy <- raster::raster(dummy)
         raster::extent(dummy) <- myExtent
-        raster::plot(dummy, col = NA, xlim = my.xlim, ylim = my.ylim, main = paste(long.name, my.title, sep = "\n"), axes = FALSE, 
-            legend = FALSE)
-        raster::plot(land, col = "grey", border = NA, add = TRUE)
-        graphics::points(data, pch = 16, col = colors)
+        raster::plot(dummy, col = NA, xlim = my.xlim, ylim = my.ylim, main = paste(paste(subcaption, long.name, sep = " "), my.title,
+            sep = "\n"), axes = FALSE, legend = FALSE)
+        raster::plot(land, col = "grey85", border = NA, add = TRUE)
+
+        # Show positive and negative biases with different symbols
+        if (i == 5) {
+            graphics::points(bias.pos, pch = 24, bg = colors.pos, cex = myCex, lwd = 0.1)
+            graphics::points(bias.neg, pch = 25, bg = colors.neg, cex = myCex, lwd = 0.1)
+        }
+
+        # all other metrics use same symbol
+        if (i != 5) {
+            graphics::points(data, pch = 21, bg = colors, cex = myCex * 1.2, lwd = 0.1)
+        }
+
         if (irregular == FALSE) {
             graphics::axis(1, labels = TRUE, tcl = 0.3)
             graphics::axis(2, labels = TRUE, tcl = 0.3, las = 2)
         }
-        plot(dummy, legend.only = TRUE, col = my.col, breaks = my.breaks, axis.args = my.axis.args, legend.args = my.legend.args, 
+        plot(dummy, legend.only = TRUE, col = my.col, breaks = my.breaks, axis.args = my.axis.args, legend.args = my.legend.args,
             legend.width = 1.5, legend.shrink = 1, font = 1)
         if (outputDir != FALSE) {
             grDevices::dev.off()
         }
     }
-    
+
 }
-utils::globalVariables("%dopar%")
+utils::globalVariables(c("%dopar%", "sitename", "Group.1"))
 
