@@ -1,9 +1,10 @@
-################################################################################ 
+################################################################################
 #' Scores for FLUXNET reference data in NetCDF format
 #' @description This function compares model output against
-#' FLUXNET measurements in NetCDF format. The performance of a model is
-#' expressed through scores that range from zero to one, where increasing values
-#' imply better performance. These scores are computed in five steps:
+#' measurements from FLUXNET or from the Baseline Surface Radiation Networking in
+#' NetCDF format. The performance of a model is expressed through scores that range
+#' from zero to one, where increasing values imply better performance. These scores
+#' are computed in five steps:
 #' \eqn{(i)} computation of a statistical metric,
 #' \eqn{(ii)} nondimensionalization,
 #' \eqn{(iii)} conversion to unit interval,
@@ -37,6 +38,9 @@
 #' @param plot.height Number that gives the plot height, e.g. 4
 #' @param numCores An integer that defines the number of cores, e.g. 2
 #' @param outputDir A string that gives the output directory, e.g. '/home/project/study'. The output will only be written if the user specifies an output directory.
+#' @param phaseMinMax A string (either 'phaseMax' or 'phaseMin') that determines
+#' whether to assess the seasonal peak as a maximum or a minimum. The latter may be appropriate for variables
+#' that tend to be negative, such as net longwave radiation or net ecosystem exchange.
 #' @return (1) Figures in PDF format that show global maps of
 #' the model data at the location of FLUXNET sites
 #' (mean, \eqn{mod.mean}; interannual-variability, \eqn{mod.iav}; month of annual cycle maximum, \eqn{mod.max.month}),
@@ -51,14 +55,24 @@
 #' site, and (iii) score values and (iv) score inputs averaged across sites.
 #' when averaging over all station.
 #' @examples
-#'
 #' library(amber)
+#' library(classInt)
 #' library(doParallel)
 #' library(foreach)
+#' library(Hmisc)
+#' library(latex2exp)
 #' library(ncdf4)
 #' library(parallel)
 #' library(raster)
-
+#' library(rgdal)
+#' library(rgeos)
+#' library(scico)
+#' library(sp)
+#' library(stats)
+#' library(utils)
+#' library(viridis)
+#' library(xtable)
+#'
 #' long.name <- 'Gross primary productivity'
 #' nc.mod <- system.file('extdata/modelRegular', 'gpp_monthly.nc', package = 'amber')
 #' nc.ref <- system.file('extdata/referenceRegular', 'gpp_FLUXNET.nc', package = 'amber')
@@ -92,8 +106,9 @@
 #' rotate.me <- FALSE
 #' irregular <- TRUE
 #' my.projection <-'+proj=ob_tran +o_proj=longlat +o_lon_p=83. +o_lat_p=42.5 +lon_0=263.'
-#' shp.filename <- system.file('extdata/ne_50m_admin_0_countries/ne_50m_admin_0_countries.shp',
-#'  package = 'amber')
+#' # shp.filename <- system.file('extdata/ne_50m_admin_0_countries/ne_50m_admin_0_countries.shp',
+#' # package = 'amber')
+#' shp.filename <- system.file("extdata/ne_110m_land/ne_110m_land.shp", package = "amber")
 #' my.xlim <- c(-171, 0) # longitude range that you wish to plot
 #' my.ylim <- c(32, 78) # latitude range that you wish to plot
 #' plot.width <- 7 # plot width
@@ -104,19 +119,19 @@
 #' unit.conv.ref, variable.unit, score.weights, rotate.me, irregular,
 #' my.projection,
 #' shp.filename, my.xlim, my.ylim)
-#' }
+#' } #donttest
 #' @export
-scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.conv.mod, unit.conv.ref, variable.unit, score.weights, 
-    rotate.me = TRUE, irregular = FALSE, my.projection = "+proj=longlat +ellps=WGS84", shp.filename = system.file("extdata/ne_110m_land/ne_110m_land.shp", 
-        package = "amber"), my.xlim = c(-180, 180), my.ylim = c(-60, 85), plot.width = 8, plot.height = 3.8, numCores = 2, 
-    outputDir = FALSE) {
-    
+scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.conv.mod, unit.conv.ref, variable.unit, score.weights,
+    rotate.me = TRUE, irregular = FALSE, my.projection = "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0",
+    shp.filename = system.file("extdata/ne_110m_land/ne_110m_land.shp", package = "amber"), my.xlim = c(-180, 180),
+    my.ylim = c(-60, 85), plot.width = 8, plot.height = 3.8, numCores = 2, outputDir = FALSE, phaseMinMax = "phaseMax") {
+
     # (I) Data preparation -----------------------------------------------------
-    
+
     # (1) Reproject data from an irregular to a regular grid if 'irregular=TRUE'
-    
+
     if (irregular == TRUE) {
-        regular <- "+proj=longlat +ellps=WGS84"
+        regular <- "+proj=longlat +datum=WGS84 +no_defs +ellps=WGS84 +towgs84=0,0,0"
         rotated <- my.projection
         # get variable name
         nc <- ncdf4::nc_open(nc.mod)
@@ -125,7 +140,7 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
         variable.name <- toupper(variable.name)  # make variable name upper-case
         ncdf4::nc_close(nc)
-        
+
         # process model data
         my.nc <- nc.mod
         nc <- ncdf4::nc_open(my.nc)
@@ -133,7 +148,7 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         lon <- ncdf4::ncvar_get(nc, "lon")
         lat <- ncdf4::ncvar_get(nc, "lat")
         time <- ncdf4::ncvar_get(nc, "time")
-        # 
+        #
         nCol <- base::length(lon[, 1])
         nRow <- base::length(lon[1, ])
         nTime <- base::length(time)
@@ -147,14 +162,14 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         end.date <- max(dates)
         start.date <- format(as.Date(start.date), "%Y-%m")
         end.date <- format(as.Date(end.date), "%Y-%m")
-        
+
         lon <- base::matrix(lon, ncol = 1)
         lat <- base::matrix(lat, ncol = 1)
-        
+
         lonLat <- base::data.frame(lon, lat)
         sp::coordinates(lonLat) <- ~lon + lat
-        raster::projection(lonLat) <- regular
-        lonLat <- sp::spTransform(lonLat, sp::CRS(rotated))
+        suppressWarnings(raster::projection(lonLat) <- regular)
+        suppressWarnings(lonLat <- sp::spTransform(lonLat, sp::CRS(rotated)))
         myExtent <- raster::extent(lonLat)
         # create an empty raster
         r <- raster::raster(ncols = nCol, nrows = nRow)  # empty raster
@@ -175,7 +190,7 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         mod <- myStack
         start.date.mod <- start.date
         end.date.mod <- end.date
-        
+
         # process reference data
         nc <- ncdf4::nc_open(nc.ref)
         variable.name.ref <- names(nc[["var"]])
@@ -193,15 +208,15 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         # convert data to data frame
         lon <- ncdf4::ncvar_get(nc, varid = "lon")
         lat <- ncdf4::ncvar_get(nc, varid = "lat")
-        
+
         lon <- base::matrix(lon, ncol = 1)
         lat <- base::matrix(lat, ncol = 1)
-        
+
         # reproject coordinates to rotated grid
         lonLat <- base::data.frame(lon, lat)
         sp::coordinates(lonLat) <- ~lon + lat
-        raster::projection(lonLat) <- regular
-        lonLat <- sp::spTransform(lonLat, sp::CRS(rotated))
+        suppressWarnings(raster::projection(lonLat) <- regular)
+        suppressWarnings(lonLat <- sp::spTransform(lonLat, sp::CRS(rotated)))
         lon <- sp::coordinates(lonLat)[, 1]
         lat <- sp::coordinates(lonLat)[, 2]
         data <- ncdf4::ncvar_get(nc, varid = variable.name.ref)
@@ -211,11 +226,11 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         site.id <- paste("site", seq(1, nrow(data), 1), sep = "_")
         rownames(data) <- site.id
         ref <- data
-        
+
     } else {
-        
+
         # (2) Process data if 'irregular=FALSE' --------------------------------
-        
+
         # model data
         nc <- ncdf4::nc_open(nc.mod)
         variable.name <- names(nc[["var"]])
@@ -253,16 +268,16 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         site.id <- paste("site", seq(1, nrow(data), 1), sep = "_")
         rownames(data) <- site.id
         ref <- data
-        
+
     }
-    
+
     #---------------------------------------------------------------------------
-    
+
     # find common time period
     start.date <- max(start.date.mod, start.date.ref)
     end.date <- min(end.date.mod, end.date.ref)
     # subset common time period for model data
-    mod <- mod[[which(format(as.Date(raster::getZ(mod)), "%Y-%m") >= start.date & format(as.Date(raster::getZ(mod)), "%Y-%m") <= 
+    mod <- mod[[which(format(as.Date(raster::getZ(mod)), "%Y-%m") >= start.date & format(as.Date(raster::getZ(mod)), "%Y-%m") <=
         end.date)]]
     # subset common time period for reference data get a sequence of common dates
     dates <- seq(from = as.Date(paste(start.date, "15", sep = "-")), to = as.Date(paste(end.date, "15", sep = "-")), by = "month")
@@ -273,8 +288,8 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     index <- seq(1, length(dates.ref), 1)
     dates.ref <- data.frame(index, dates.ref)
     colnames(dates.ref) <- c("index", "dates")
-    ## merge common dates and reference dates the index will be used to subset the reference data set if the time covered by
-    ## the reference data exceeds the time covered by the model data
+    ## merge common dates and reference dates the index will be used to subset the reference data set if the time covered by the
+    ## reference data exceeds the time covered by the model data
     column.id <- merge(dates.ref, common.dates, by = "dates")
     column.id <- column.id$index
     ref <- data.frame(ref[2 + column.id])  # 2+ because column 1 and 2 are are lon and lat
@@ -282,10 +297,10 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     ref <- intFun.site.points(lon, lat, ref)
     # compute statistics for each site by extracting model data for each site
     if (rotate.me == TRUE) {
-        mod <- raster::rotate(mod)
+        suppressWarnings(mod <- raster::rotate(mod))
     }
     mod <- mod * unit.conv.mod
-    mod <- raster::extract(mod, ref, method = "bilinear")
+    mod <- raster::extract(mod, ref, method = "simple")
     mod <- data.frame(mod)
     rownames(mod) <- site.id
     ref <- as.data.frame(ref)
@@ -297,8 +312,8 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     meta.data.mod <- paste(variable.name, mod.id, "from", start.date, "to", end.date, sep = "_")
     meta.data.ref <- paste(variable.name, ref.id, "from", start.date, "to", end.date, sep = "_")
     meta.data.com <- paste(variable.name, mod.id, "vs", ref.id, "from", start.date, "to", end.date, sep = "_")
-    # statistical analysis the time period of the reference data varies among sites some sites are located in the model
-    # ocean exclude data in one data set that is not available in the other data set
+    # statistical analysis the time period of the reference data varies among sites some sites are located in the model ocean
+    # exclude data in one data set that is not available in the other data set
     mask.mod <- (mod - mod + 1)
     mask.ref <- (ref - ref + 1)
     mask <- mask.mod * mask.ref
@@ -317,7 +332,8 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     weights <- ref.mean  # weights used for spatial integral
     bias <- mod.mean - ref.mean  # time mean
     ref.sd <- apply(ref, 2, sd, na.rm = TRUE)  # standard deviation of reference data
-    epsilon_bias <- abs(bias)/ref.sd  # relative error
+    epsilon_bias <- abs(bias)/ref.sd
+    epsilon_bias[epsilon_bias == Inf] <- NA  # relative error
     bias.score <- exp(-epsilon_bias)  # bias score as a function of space
     S_bias_not.weighted <- mean(bias.score, na.rm = TRUE)  # scalar score (not weighted)
     # calculate the weighted scalar score
@@ -328,14 +344,16 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     mod.mean.scalar <- mean(mod.mean, na.rm = TRUE)  # global mean value
     ref.mean.scalar <- mean(ref.mean, na.rm = TRUE)  # global mean value
     bias.scalar <- mean(bias, na.rm = TRUE)  # global mean value
+    bias.scalar.rel <- (mod.mean.scalar - ref.mean.scalar)/abs(ref.mean.scalar) * 100
     ref.sd.scalar <- mean(ref.sd, na.rm = TRUE)
-    epsilon_bias.scalar <- mean(epsilon_bias, na.rm = TRUE)
+    epsilon_bias.scalar <- stats::median(epsilon_bias, na.rm = TRUE)
     # (2) root mean square error (rmse)
     rmse <- mapply(intFun.rmse, mod, ref)  # compute rmse
     mod.anom <- data.frame(apply(mod, 2, intFun.anom))  # compute anomalies
     ref.anom <- data.frame(apply(ref, 2, intFun.anom))  # compute anomalies
     crmse <- mapply(intFun.crmse, mod.anom, ref.anom)
-    epsilon_rmse <- crmse/ref.sd  # relative error
+    epsilon_rmse <- crmse/ref.sd
+    epsilon_rmse[epsilon_rmse == Inf] <- NA  # relative error
     rmse.score <- exp(-epsilon_rmse)  # rmse score as a function of space
     S_rmse_not.weighted <- mean(rmse.score, na.rm = TRUE)  # scalar score (not weighted)
     # calculate the weighted scalar score
@@ -345,7 +363,7 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     # compute global mean values of score input(s)
     rmse.scalar <- mean(rmse, na.rm = TRUE)  # global mean value
     crmse.scalar <- mean(crmse, na.rm = TRUE)  # global mean value
-    epsilon_rmse.scalar <- mean(epsilon_rmse, na.rm = TRUE)
+    epsilon_rmse.scalar <- stats::median(epsilon_rmse, na.rm = TRUE)
     # (3) phase shift make monthly means month index is the same for mod and ref, given the temporal subset made above
     month <- format(as.Date(paste(dates, "15", sep = "-")), format = "%m")
     month <- as.numeric(month)
@@ -364,9 +382,21 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     # drop 'month' column
     mod.clim.mly <- subset(mod.clim.mly, select = -c(month))
     ref.clim.mly <- subset(ref.clim.mly, select = -c(month))
-    # find month with max value
-    mod.max.month <- apply(mod.clim.mly, 2, which.max)
-    ref.max.month <- apply(ref.clim.mly, 2, which.max)
+    # find month of seasonal peak
+
+    # In most cases, we are interested in the timing of the seasonal maximum value
+
+    # In some cases, however, the seasonal peak is a minimum, e.g. NEE = RECO - GPP
+    if (phaseMinMax == "phaseMax") {
+        mod.max.month <- apply(mod.clim.mly, 2, which.max)
+        ref.max.month <- apply(ref.clim.mly, 2, which.max)
+    }
+
+    if (phaseMinMax == "phaseMin") {
+        mod.max.month <- apply(mod.clim.mly, 2, which.min)
+        ref.max.month <- apply(ref.clim.mly, 2, which.min)
+    }
+
     mod.max.month <- as.numeric(mod.max.month)
     mod.max.month <- data.frame(as.numeric(mod.max.month))
     ref.max.month <- data.frame(as.numeric(ref.max.month))
@@ -384,19 +414,19 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     mod.max.month.scalar <- mean(mod.max.month[, 1], na.rm = TRUE)  # global mean value
     ref.max.month.scalar <- mean(ref.max.month[, 1], na.rm = TRUE)  # global mean value
     # (4) interannual variability
-    
-    # This approach assumes that all data start in Jan all months after the last Dec will be dropped if data does not end in
-    # Dec
+
+    # This approach assumes that all data start in Jan all months after the last Dec will be dropped if data does not end in Dec
     mod.anom <- intFun.anom.mly(mod, mod.clim.mly.mm)
     ref.anom <- intFun.anom.mly(ref, ref.clim.mly.mm)
     mod.iav <- apply(mod.anom, 2, intFun.iav)
     ref.iav <- apply(ref.anom, 2, intFun.iav)
-    
+
     # set values close to zero to NA
     ref.iav.na <- ref.iav
     ref.iav.na[ref.iav.na < 10^(-5)] <- NA
-    
-    epsilon_iav <- abs((mod.iav - ref.iav))/ref.iav.na  # I changed Eq. 26 so that epsilon_iav > =  0
+
+    epsilon_iav <- abs((mod.iav - ref.iav))/ref.iav.na
+    epsilon_iav[epsilon_iav == Inf] <- NA  # I changed Eq. 26 so that epsilon_iav > =  0
     iav.score <- exp(-epsilon_iav)  # iav score as a function of space
     S_iav_not.weighted <- mean(iav.score, na.rm = TRUE)  # scalar score (not weighted)
     # calculate the weighted scalar score
@@ -406,8 +436,8 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     # compute global mean values of score input(s)
     mod.iav.scalar <- mean(mod.iav, na.rm = TRUE)  # global mean value
     ref.iav.scalar <- mean(ref.iav, na.rm = TRUE)  # global mean value
-    epsilon_iav.scalar <- mean(epsilon_iav, na.rm = TRUE)  # global mean value
-    
+    epsilon_iav.scalar <- stats::median(epsilon_iav, na.rm = TRUE)  # global mean value
+
     # (5) dist
     mod.sigma.scalar <- sd(mod.mean, na.rm = TRUE)  # standard deviation of period mean data
     ref.sigma.scalar <- sd(ref.mean, na.rm = TRUE)  # standard deviation of period mean data
@@ -429,7 +459,7 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     S_phase <- S_phase_not.weighted
     S_iav <- S_iav_not.weighted
     # weight importance of statisitcal metrics and compute overall score
-    S_overall <- (w.bias * S_bias + w.rmse * S_rmse + w.phase * S_phase + w.iav * S_iav + w.dist * S_dist)/(w.bias + w.rmse + 
+    S_overall <- (w.bias * S_bias + w.rmse * S_rmse + w.phase * S_phase + w.iav * S_iav + w.dist * S_dist)/(w.bias + w.rmse +
         w.phase + w.iav + w.dist)
     scores <- data.frame(variable.name, ref.id, S_bias, S_rmse, S_phase, S_iav, S_dist, S_overall)
     scores_not.weighted <- scores
@@ -438,11 +468,11 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     S_rmse <- S_rmse_weighted
     S_phase <- S_phase_weighted
     S_iav <- S_iav_weighted
-    S_overall <- (w.bias * S_bias + w.rmse * S_rmse + w.phase * S_phase + w.iav * S_iav + w.dist * S_dist)/(w.bias + w.rmse + 
+    S_overall <- (w.bias * S_bias + w.rmse * S_rmse + w.phase * S_phase + w.iav * S_iav + w.dist * S_dist)/(w.bias + w.rmse +
         w.phase + w.iav + w.dist)
     scores <- data.frame(variable.name, ref.id, S_bias, S_rmse, S_phase, S_iav, S_dist, S_overall)
     scores_weighted <- scores
-    # 
+    #
     scores <- rbind(scores_not.weighted, scores_weighted)
     rownames(scores) <- c("not.weighted", "weighted")
     if (outputDir != FALSE) {
@@ -457,9 +487,9 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         utils::write.table(all.score.values, paste(outputDir, "/", "allscorevalues", "-", variable.name, "-", ref.id, sep = ""))
     }
     # selected score inputs
-    scoreinputs <- data.frame(long.name, variable.name, ref.id, variable.unit, mod.mean.scalar, ref.mean.scalar, bias.scalar, 
-        ref.sd.scalar, epsilon_bias.scalar, S_bias_not.weighted, rmse.scalar, crmse.scalar, ref.sd.scalar, epsilon_rmse.scalar, 
-        S_rmse_not.weighted, mod.max.month.scalar, ref.max.month.scalar, phase.scalar, S_phase_not.weighted, mod.iav.scalar, 
+    scoreinputs <- data.frame(long.name, variable.name, ref.id, variable.unit, mod.mean.scalar, ref.mean.scalar, bias.scalar,
+        bias.scalar.rel, ref.sd.scalar, epsilon_bias.scalar, S_bias_not.weighted, rmse.scalar, crmse.scalar, ref.sd.scalar, epsilon_rmse.scalar,
+        S_rmse_not.weighted, mod.max.month.scalar, ref.max.month.scalar, phase.scalar, S_phase_not.weighted, mod.iav.scalar,
         ref.iav.scalar, epsilon_iav.scalar, S_iav_not.weighted, mod.sigma.scalar, ref.sigma.scalar, sigma, R, S_dist)
     if (outputDir != FALSE) {
         utils::write.table(scoreinputs, paste(outputDir, "/", "scoreinputs", "_", meta.data.com, sep = ""))
@@ -477,11 +507,11 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     mmi.max.month <- c(1, 12, 1)
     mmi.iav <- intFun.min.max.int.mod.ref(mod.iav, ref.iav)
     # add metadata:
-    
+
     # 1. figure title (e.g. Mean_nee_ModID_123_from_1982-01_to_2008-12)
-    
+
     # 2. min, max, interval used in legend (e.g. 0, 1, 0.1)
-    
+
     # 3. legend bar text (e.g. 'score (-)')
     attr(mod.mean, "metadata") <- list(paste("Mean", meta.data.mod, sep = "_"), mmi.mean, variable.unit)
     attr(ref.mean, "metadata") <- list(paste("Mean", meta.data.ref, sep = "_"), mmi.mean, variable.unit)
@@ -497,15 +527,15 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
     attr(ref.iav, "metadata") <- list(paste("IAV", meta.data.ref, sep = "_"), mmi.iav, variable.unit)
     attr(iav.score, "metadata") <- list(paste("IAV_score", meta.data.com, sep = "_"), mmi.iav.score, "score (-)")
     # write data to file
-    stat.metric <- data.frame(lon, lat, mod.mean, ref.mean, bias, bias.score, crmse, rmse.score, mod.max.month, ref.max.month, 
+    stat.metric <- data.frame(lon, lat, mod.mean, ref.mean, bias, bias.score, crmse, rmse.score, mod.max.month, ref.max.month,
         phase, phase.score, mod.iav, ref.iav, iav.score)
-    colnames(stat.metric) <- c("lon", "lat", "mod.mean", "ref.mean", "bias", "bias.score", "crmse", "rmse.score", "mod.max.month", 
+    colnames(stat.metric) <- c("lon", "lat", "mod.mean", "ref.mean", "bias", "bias.score", "crmse", "rmse.score", "mod.max.month",
         "ref.max.month", "phase", "phase.score", "mod.iav", "ref.iav", "iav.score")
-    my.filename <- paste(variable.name, "FLUXNET", sep = "_")
+    my.filename <- paste(variable.name, ref.id, sep = "_")
     if (outputDir != FALSE) {
         utils::write.table(stat.metric, paste(outputDir, my.filename, sep = "/"))
     }
-    
+
     land <- intFun.coast(my.xlim, my.ylim, my.projection, shp.filename)  # reproject coastline
     # loop through all layers of the raster stack
     lon2 <- stat.metric$lon
@@ -538,30 +568,30 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         my.col <- viridis::viridis(n = length(my.breaks) - 1, direction = -1)
         my.col.bias <- scico::scico(n = length(my.breaks) - 1, palette = "vik")
         my.col.phase <- grDevices::rainbow(n = length(my.breaks) - 1)
-        if (i == 5) 
+        if (i == 5)
             {
                 my.col <- my.col.bias
             }  # divergent color scheme for bias plots
-        if (i == 9) 
+        if (i == 9)
             {
                 my.col <- my.col.phase
             }  # circular color scheme for phase
-        if (i == 10) 
+        if (i == 10)
             {
                 my.col <- my.col.phase
             }  # circular color scheme for phase
-        if (i == 11) 
+        if (i == 11)
             {
                 my.col <- my.col.phase
             }  # circular color scheme for phase
-        
+
         my.axis.args <- list(at = my.labels, labels = my.labels, cex.axis = 1)
         my.legend.args <- list(text = legend.bar.text, side = 2, font = 1, line = 1, cex = 1)
-        
+
         colors <- cut(values$values, breaks = my.breaks, labels = my.col, include.lowest = TRUE)
         colors <- toString(colors)
         colors <- unlist(strsplit(colors, split = ", "))
-        
+
         # plot
         oldpar <- graphics::par(mfrow = c(1, 2))
         on.exit(graphics::par(oldpar))
@@ -582,7 +612,7 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
         dummy <- matrix(dummy, nrow = 180)
         dummy <- raster::raster(dummy)
         raster::extent(dummy) <- myExtent
-        raster::plot(dummy, col = NA, xlim = my.xlim, ylim = my.ylim, main = paste(long.name, my.title, sep = "\n"), axes = FALSE, 
+        raster::plot(dummy, col = NA, xlim = my.xlim, ylim = my.ylim, main = paste(long.name, my.title, sep = "\n"), axes = FALSE,
             legend = FALSE)
         raster::plot(land, col = "grey", border = NA, add = TRUE)
         graphics::points(data, pch = 16, col = colors)
@@ -590,12 +620,12 @@ scores.fluxnet.nc <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.co
             graphics::axis(1, labels = TRUE, tcl = 0.3)
             graphics::axis(2, labels = TRUE, tcl = 0.3, las = 2)
         }
-        plot(dummy, legend.only = TRUE, col = my.col, breaks = my.breaks, axis.args = my.axis.args, legend.args = my.legend.args, 
+        raster::plot(dummy, legend.only = TRUE, col = my.col, breaks = my.breaks, axis.args = my.axis.args, legend.args = my.legend.args,
             legend.width = 1.5, legend.shrink = 1, font = 1)
         if (outputDir != FALSE) {
             grDevices::dev.off()
         }
     }
-    
+
 }
 
