@@ -1,22 +1,22 @@
-################################################################################ 
+################################################################################
 #' Scores for gridded reference data that do not have a varying time dimension
 #' @description This function compares model output against remote-sensing
 #' based reference data that do not vary in time. The performance of a model is
 #' expressed through a score that ranges from zero to one, where increasing values
 #' imply better performance.
-#' Contrary to the function \link{scores.grid.time}, only one score is computed
-#' (spatial distribution score, \eqn{S_{dist}}) since the reference data does
+#' Contrary to the function \link{scores.grid.time}, only two scores are computed
+#' (bias score \eqn{S_{bias}} and spatial distribution score, \eqn{S_{dist}}) since the reference data do
+#' not vary with time. Contrary to \link{scores.grid.time}, the bias is relative to the absolute reference mean
+#' value rather than the reference standard deviation. Again, this is because the reference data do
 #' not vary with time:
 #'
-#' \eqn{\sigma=\sigma_{\overline{v_{mod}}}/\sigma_{\overline{v_{ref}}}}
+#' \eqn{(i) \ bias(\lambda, \phi)=\overline{v_{mod}}(\lambda, \phi)-\overline{v_{ref}}(\lambda, \phi)}
 #'
-#' \eqn{S_{dist}=2(1+R)/(\sigma+\frac{1}{\sigma})^{2}}
+#' \eqn{(ii) \ \varepsilon_{bias}=|bias(\lambda, \phi)|/|\overline{v_{ref}}(\lambda, \phi)|}
 #'
-#' where \eqn{\sigma_{\overline{v_{mod}}}} and \eqn{\sigma_{\overline{v_{ref}}}} are the
-#' standard deviation of the time mean values from the model and reference data,
-#' and \eqn{R} is the spatial correlation coefficient of
-#' \eqn{\overline{v_{ref}}(\lambda, \phi)} and
-#' \eqn{\overline{v_{mod}}(\lambda, \phi)}.
+#' \eqn{(iii) \ s_{bias}(\lambda, \phi)=e^{-\varepsilon_{bias}(\lambda, \phi)}}
+#'
+#' \eqn{(iv) \ S_{bias}=\overline{\overline{s_{bias}}}}
 #'
 #' @param long.name A string that gives the full name of the variable, e.g. 'Gross primary productivity'
 #' @param nc.mod A string that gives the path and name of the netcdf file that contains the model output, e.g. '/home/model_gpp.nc'
@@ -37,9 +37,10 @@
 #' @param irregular Logical. If TRUE the data are converted from an irregular to a regular grid. Default is FALSE.
 #' @param my.projection A string that defines the projection of the irregular grid
 #' @param numCores An integer that defines the number of cores, e.g. 2
-#' @param period An R obect that gives the period over which to average the model data, e.g. c('1980-01', '2017-12')
+#' @param period An R object that gives the period over which to average the model data, e.g. c('1980-01', '2017-12')
 #' @param timeInt A string that gives the time interval of the model data, e.g. 'month' or 'year'
 #' @param outputDir A string that gives the output directory, e.g. '/home/project/study'. The output will only be written if the user specifies an output directory.
+#' @param variable.name A string with the variable name, e.g. 'GPP'. If FALSE, the variable name stored in the NetCDF file will be used instead. Default is FALSE.
 #'
 #' @return (1) A list that contains three elements. The first element is a
 #' raster stack with model data
@@ -52,18 +53,28 @@
 #' The content of the list can be plotted using \link{plotGrid}.
 #'
 #' (2) NetCDF files for each of the statistical variables listed above.
-# 
+#
 #' (3) Three text files: (i) score values and (ii) score inputs averaged across
 #' the entire grid, and (iii) score values for each individual grid cell.
 #'
 #' @examples
-#'
 #' library(amber)
+#' library(classInt)
 #' library(doParallel)
 #' library(foreach)
+#' library(Hmisc)
+#' library(latex2exp)
 #' library(ncdf4)
 #' library(parallel)
 #' library(raster)
+#' library(rgdal)
+#' library(rgeos)
+#' library(scico)
+#' library(sp)
+#' library(stats)
+#' library(utils)
+#' library(viridis)
+#' library(xtable)
 #'
 #' # (1) Global plots on a regular grid
 #' long.name <- 'Soil Carbon'
@@ -81,20 +92,6 @@
 #' plotGrid(long.name, plot.me)
 #'
 #' \donttest{
-#' # Additional parameters:
-#' score.weights <- c(1,2,1,1,1) # score weights of S_bias, S_rmse, S_phase, S_iav, S_dist
-#' outlier.factor <- 1000
-#' irregular <- FALSE
-#' my.projection <- '+proj=ob_tran +o_proj=longlat +o_lon_p=83. +o_lat_p=42.5 +lon_0=263.'
-#' numCores <- 2
-#' period <- c('1980-01', '2017-12') # period over which to average the model data
-#'
-#' plot.me <- scores.grid.notime(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.conv.mod,
-#' unit.conv.ref, variable.unit, score.weights, outlier.factor, irregular, my.projection,
-#' numCores, period)
-#' plotGrid(long.name, plot.me)
-#'
-#'
 #' # (2) Regional plots on a rotated grid
 #' long.name <- 'Soil Carbon'
 #' nc.mod <- system.file('extdata/modelRotated', 'cSoil_monthly.nc', package = 'amber')
@@ -118,8 +115,9 @@
 #' # Plot results:
 #' irregular <- TRUE # data is on an irregular grid
 #' my.projection <- '+proj=ob_tran +o_proj=longlat +o_lon_p=83. +o_lat_p=42.5 +lon_0=263.'
-#' shp.filename <- system.file('extdata/ne_50m_admin_0_countries/ne_50m_admin_0_countries.shp',
-#'  package = 'amber')
+#' # shp.filename <- system.file('extdata/ne_50m_admin_0_countries/ne_50m_admin_0_countries.shp',
+#' #  package = 'amber')
+#' shp.filename <- system.file("extdata/ne_110m_land/ne_110m_land.shp", package = "amber")
 #' my.xlim <- c(-171, 0) # longitude range that you wish to plot
 #' my.ylim <- c(32, 78) # latitude range that you wish to plot
 #' plot.width <- 7 # plot width
@@ -127,30 +125,32 @@
 #'
 #' plotGrid(long.name, plot.me, irregular, my.projection,
 #' shp.filename, my.xlim, my.ylim, plot.width, plot.height)
-#' }
+#' } #donttest
 #' @export
-scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.conv.mod, unit.conv.ref, variable.unit, score.weights = c(1, 
-    2, 1, 1, 1), outlier.factor = 1000, irregular = FALSE, my.projection = "+proj=ob_tran +o_proj=longlat +o_lon_p=83. +o_lat_p=42.5 +lon_0=263.", 
-    numCores = 2, period = c("1980-01", "2017-12"), timeInt = "month", outputDir = FALSE) {
-    
+scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.conv.mod, unit.conv.ref, variable.unit, score.weights = c(1,
+    2, 1, 1, 1), outlier.factor = 1000, irregular = FALSE, my.projection = "+proj=ob_tran +o_proj=longlat +o_lon_p=83. +o_lat_p=42.5 +lon_0=263.",
+    numCores = 2, period = c("1980-01", "2017-12"), timeInt = "month", outputDir = FALSE, variable.name = FALSE) {
+
     #---------------------------------------------------------------------------
-    
+
     # (I) Data preparation
-    
+
     #---------------------------------------------------------------------------
-    
+
     # (1) Reproject data from an irregular to a regular grid if 'irregular=TRUE'
-    
+
     if (irregular == TRUE) {
         regular <- "+proj=longlat +ellps=WGS84"
         rotated <- my.projection
         # get variable name
-        nc <- ncdf4::nc_open(nc.mod)
-        variable.name <- base::names(nc[["var"]])
-        variable.name <- variable.name[1]
-        variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
-        variable.name <- toupper(variable.name)  # make variable name upper-case
-        ncdf4::nc_close(nc)
+        if (variable.name == FALSE) {
+            nc <- ncdf4::nc_open(nc.mod)
+            variable.name <- base::names(nc[["var"]])
+            variable.name <- variable.name[1]
+            variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
+            variable.name <- toupper(variable.name)  # make variable name upper-case
+            ncdf4::nc_close(nc)
+        }
         # get data for both, model and reference data
         modRef <- c(nc.mod, nc.ref)
         for (id in 1:length(modRef)) {
@@ -160,7 +160,7 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
             lon <- ncdf4::ncvar_get(nc, "lon")
             lat <- ncdf4::ncvar_get(nc, "lat")
             time <- ncdf4::ncvar_get(nc, "time")
-            # 
+            #
             nCol <- base::length(lon[, 1])
             nRow <- base::length(lon[1, ])
             nTime <- base::length(time)
@@ -174,14 +174,14 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
             end.date <- max(dates)
             start.date <- format(as.Date(start.date), "%Y-%m")
             end.date <- format(as.Date(end.date), "%Y-%m")
-            
+
             lon <- base::matrix(lon, ncol = 1)
             lat <- base::matrix(lat, ncol = 1)
-            
+
             lonLat <- base::data.frame(lon, lat)
             sp::coordinates(lonLat) <- ~lon + lat
             raster::projection(lonLat) <- regular
-            lonLat <- sp::spTransform(lonLat, sp::CRS(rotated))
+            suppressWarnings(lonLat <- sp::spTransform(lonLat, sp::CRS(rotated)))
             myExtent <- raster::extent(lonLat)
             # create an empty raster
             r <- raster::raster(ncols = nCol, nrows = nRow)  # empty raster
@@ -208,75 +208,72 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
         # model data are averaged over a period since the reference data set has no time dimension
         start.date <- period[1]
         end.date <- period[2]
-        
-        mod <- mod[[which(format(as.Date(raster::getZ(mod)), "%Y-%m") >= start.date & format(as.Date(raster::getZ(mod)), 
-            "%Y-%m") <= end.date)]]
+
+        mod <- mod[[which(format(as.Date(raster::getZ(mod)), "%Y-%m") >= start.date & format(as.Date(raster::getZ(mod)), "%Y-%m") <=
+            end.date)]]
         mod <- raster::mean(mod, na.rm = TRUE)
-        
-        nc <- ncdf4::nc_open(nc.mod)
-        variable.name <- names(nc[["var"]])
-        ncdf4::nc_close(nc)
-        variable.name <- variable.name[1]
-        variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
-        variable.name <- toupper(variable.name)  # make variable name upper-case
-        
+
         # unit conversion if appropriate
         mod <- mod * unit.conv.mod
         ref <- ref * unit.conv.ref
-        
+
         # get layer names
         mod.names <- names(mod)
         ref.names <- names(ref)
     } else {
-        
+
         #-----------------------------------------------------------------------
-        
+
         # (2) Process data if 'irregular=FALSE'
-        
+
         #-----------------------------------------------------------------------
-        
+
         # model data are averaged over a period since the reference data set has no time dimension
         start.date <- period[1]
         end.date <- period[2]
-        
+
         mod <- raster::brick(nc.mod)
-        mod <- raster::rotate(mod)
+        suppressWarnings(mod <- raster::rotate(mod))
         dates.mod <- raster::getZ(mod)
-        
+
         mod <- raster::setZ(mod, dates.mod, name = "date")
-        mod <- mod[[which(format(as.Date(raster::getZ(mod)), "%Y-%m") >= start.date & format(as.Date(raster::getZ(mod)), 
-            "%Y-%m") <= end.date)]]
+        mod <- mod[[which(format(as.Date(raster::getZ(mod)), "%Y-%m") >= start.date & format(as.Date(raster::getZ(mod)), "%Y-%m") <=
+            end.date)]]
+
         mod <- raster::mean(mod, na.rm = TRUE)
-        
-        nc <- ncdf4::nc_open(nc.mod)
-        variable.name <- names(nc[["var"]])
-        ncdf4::nc_close(nc)
-        variable.name <- variable.name[length(variable.name)]  # take the last variable (relevant for CanESM5)
-        variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
-        variable.name <- toupper(variable.name)  # make variable name upper-case
-        
+
+        # get variable name
+        if (variable.name == FALSE) {
+            nc <- ncdf4::nc_open(nc.mod)
+            variable.name <- names(nc[["var"]])
+            ncdf4::nc_close(nc)
+            variable.name <- variable.name[length(variable.name)]  # take the last variable (relevant for CanESM5)
+            variable.name <- ifelse(variable.name == "burntFractionAll", "burnt", variable.name)  # rename burntFractionAll to shorter name
+            variable.name <- toupper(variable.name)  # make variable name upper-case
+        }
+
         # reference data
         ref <- raster::raster(nc.ref)  # this reference data has no time dimension
-        ref <- raster::rotate(ref)
+        suppressWarnings(ref <- raster::rotate(ref))
         # unit conversion if appropriate
         mod <- mod * unit.conv.mod
         ref <- ref * unit.conv.ref
-        
+
         # get layer names
         mod.names <- names(mod)
         ref.names <- names(ref)
     }
-    
+
     #---------------------------------------------------------------------------
-    
+
     # 1.3 Remaining part applies to both regular and irregular gridded data
-    
+
     #---------------------------------------------------------------------------
-    
+
     # Make a string that summarizes metadata. This will be added to each netcdf file (longname).
-    
+
     # The string can then be accessed like this: names(raster(file.nc))
-    
+
     meta.data.mod <- paste(variable.name, mod.id, "from", start.date, "to", end.date, sep = "_")
     meta.data.ref <- paste(variable.name, ref.id, "from", start.date, "to", end.date, sep = "_")
     meta.data.com <- paste(variable.name, mod.id, "vs", ref.id, "from", start.date, "to", end.date, sep = "_")
@@ -300,34 +297,53 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
     ref <- ref * ref.mask_outliers
     names(ref) <- ref.names
     ref.outlier.points <- intFun.grid.outliers.points(ref.mean, outlier.neg, outlier.pos)
-    
+
     #---------------------------------------------------------------------------
-    
+
     # II Statistical analysis
-    
+
     #---------------------------------------------------------------------------
-    
+
     # (1) Bias
-    
+
     #---------------------------------------------------------------------------
+    # create a mask to excludes all grid cells that the model and reference data do not have in common.  This mask varies in
+    # time.
+    mask <- (mod * ref)
+    mask <- mask - mask + 1
+    mod <- mod * mask
+    # names(mod) <- mod.names # this adds the corresponding dates
+    ref <- ref * mask
+    # names(ref) <- ref.names # this adds the corresponding dates now mod and ref are based on the same grid cells
+
+    #---------------------------------------------------------------------------
+
     mod.mean <- mod  # data is already time mean
     ref.mean <- ref  # data is already time mean
+    weights <- ref  # weights used for spatial integral
     bias <- mod.mean - ref.mean  # time mean
-    S_bias_not.weighted <- NA
-    S_bias_weighted <- NA
+    epsilon_bias <- abs(bias)/abs(ref)
+    epsilon_bias[epsilon_bias == Inf] <- NA  # relative error wrt ref mean
+    bias.score <- exp(-epsilon_bias)  # bias score as a function of space
+    S_bias_not.weighted <- mean(raster::getValues(bias.score), na.rm = TRUE)  # scalar score (not weighted)
+    # calculate the weighted scalar score
+    a <- bias.score * weights  # this is a raster
+    b <- raster::cellStats(a, "sum")  # this is a scalar, the sum up all values
+    S_bias_weighted <- b/raster::cellStats(weights, "sum")  # scalar score (weighted)
     # Compute global mean values of score input(s) The mask ensures that mod and ref are based on same grid cells.
     mask <- (mod.mean * ref.mean)
     mask <- mask - mask + 1
     mod.mean.scalar <- mean(raster::getValues(mask * mod.mean), na.rm = TRUE)  # global mean value
     ref.mean.scalar <- mean(raster::getValues(mask * ref.mean), na.rm = TRUE)  # global mean value
     bias.scalar <- mean(raster::getValues(bias), na.rm = TRUE)  # global mean value
+    bias.scalar.rel <- (mod.mean.scalar - ref.mean.scalar)/abs(ref.mean.scalar) * 100
     ref.sd.scalar <- NA  # global mean value
-    epsilon_bias.scalar <- NA  # global mean value
-    
+    epsilon_bias.scalar <- stats::median(raster::getValues(epsilon_bias), na.rm = TRUE)  # global mean value
+
     #---------------------------------------------------------------------------
-    
+
     # (5) dist
-    
+
     #---------------------------------------------------------------------------
     mod.sigma.scalar <- sd(raster::getValues(mod.mean), na.rm = TRUE)  # standard deviation of period mean data
     ref.sigma.scalar <- sd(raster::getValues(ref.mean), na.rm = TRUE)  # standard deviation of period mean data
@@ -337,19 +353,19 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
     reg <- stats::lm(y ~ x)
     R <- sqrt(summary(reg)$r.squared)
     S_dist <- 2 * (1 + R)/(sigma + 1/sigma)^2  # weighting does not apply
-    
+
     #---------------------------------------------------------------------------
-    
+
     # Scores
-    
+
     #---------------------------------------------------------------------------
-    
+
     S_bias <- S_bias_not.weighted
     S_rmse <- NA
     S_phase <- NA
     S_iav <- NA
     # weight importance of statisitcal metrics and compute overall score
-    S_overall <- S_dist
+    S_overall <- (S_bias + S_dist)/2
     scores <- data.frame(variable.name, ref.id, S_bias, S_rmse, S_phase, S_iav, S_dist, S_overall)
     scores_not.weighted <- scores
     # weighted (except for S_dist)
@@ -358,22 +374,27 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
     S_phase <- NA
     S_iav <- NA
     S_overall <- S_dist
+    S_overall <- (S_bias + S_dist)/2
+
     scores <- data.frame(variable.name, ref.id, S_bias, S_rmse, S_phase, S_iav, S_dist, S_overall)
     scores_weighted <- scores
-    # 
+    #
     scores <- rbind(scores_not.weighted, scores_weighted)
     rownames(scores) <- c("not.weighted", "weighted")
     if (outputDir != FALSE) {
         utils::write.table(scores, paste(outputDir, "/", "scorevalues", "_", meta.data.com, sep = ""))
     }
-    # get all score values in case you want to compare this run against another run having all values will enable you to
-    # conduct a significance test
-    dist.score <- raster::getValues(mask * S_dist)
-    all.score.values <- data.frame(NA, NA, NA, NA, dist.score)
+    # Get all score values in case you want to compare this run against another run
+
+    # Having all values will enable you to conduct a significance test
+
+    bias.score.values <- raster::getValues(bias.score)
+    dist.score.values <- raster::getValues(mask * S_dist)
+    all.score.values <- data.frame(bias.score.values, NA, NA, NA, dist.score.values)
     colnames(all.score.values) <- c("bias.score", "rmse.score", "phase.score", "iav.score", "dist.score")
     all.score.values[is.na(all.score.values)] <- NA  # converts all NaN to NA
     if (outputDir != FALSE) {
-        utils::write.table(all.score.values, paste(outputDir, "/", "allscorevalues", variable.name, "-", ref.id, sep = ""))
+        utils::write.table(all.score.values, paste(outputDir, "/", "allscorevalues", "-", variable.name, "-", ref.id, sep = ""))
     }
     # selected score inputs
     rmse.scalar <- NA
@@ -388,10 +409,10 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
     S_phase_not.weighted <- NA
     epsilon_iav.scalar <- NA
     S_iav_not.weighted <- NA
-    
-    scoreinputs <- data.frame(long.name, variable.name, ref.id, variable.unit, mod.mean.scalar, ref.mean.scalar, bias.scalar, 
-        ref.sd.scalar, epsilon_bias.scalar, S_bias_not.weighted, rmse.scalar, crmse.scalar, ref.sd.scalar, epsilon_rmse.scalar, 
-        S_rmse_not.weighted, mod.max.month.scalar, ref.max.month.scalar, phase.scalar, S_phase_not.weighted, mod.iav.scalar, 
+
+    scoreinputs <- data.frame(long.name, variable.name, ref.id, variable.unit, mod.mean.scalar, ref.mean.scalar, bias.scalar,
+        bias.scalar.rel, ref.sd.scalar, epsilon_bias.scalar, S_bias_not.weighted, rmse.scalar, crmse.scalar, ref.sd.scalar, epsilon_rmse.scalar,
+        S_rmse_not.weighted, mod.max.month.scalar, ref.max.month.scalar, phase.scalar, S_phase_not.weighted, mod.iav.scalar,
         ref.iav.scalar, epsilon_iav.scalar, S_iav_not.weighted, mod.sigma.scalar, ref.sigma.scalar, sigma, R, S_dist)
     if (outputDir != FALSE) {
         utils::write.table(scoreinputs, paste(outputDir, "/", "scoreinputs", "_", meta.data.com, sep = ""))
@@ -399,19 +420,21 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
     # function that returns the min, max, and interval used in legend function min.max.int only requires one input apply the
     # functions min.max.int and min.max.int.mod.ref min.max.int
     mmi.bias <- intFun.min.max.int.bias(bias)
+    mmi.bias.score <- c(0, 1, 0.1)
     mmi.mean <- intFun.min.max.int.mod.ref(mod.mean, ref.mean)
-    # add metadata: 1. filename (e.g. nee_mod.mean.nc), 2. figure title (e.g.  Mean_nee_ModID_123_from_1982-01_to_2008-12),
-    # 3.  min, max, interval used in legend (e.g. 0, 1, 0.1), 4.  legend bar text (e.g. 'score (-)')
-    raster::metadata(mod.mean) <- list(paste(variable.name, ref.id, "mod_mean", sep = "_"), paste("Mean", meta.data.mod, 
-        sep = "_"), mmi.mean, variable.unit)
-    raster::metadata(ref.mean) <- list(paste(variable.name, ref.id, "ref_mean", sep = "_"), paste("Mean", meta.data.ref, 
-        sep = "_"), mmi.mean, variable.unit)
-    raster::metadata(bias) <- list(paste(variable.name, ref.id, "bias", sep = "_"), paste("Bias", meta.data.com, sep = "_"), 
+    # add metadata: 1. filename (e.g. nee_mod.mean.nc), 2. figure title (e.g.  Mean_nee_ModID_123_from_1982-01_to_2008-12), 3.
+    # min, max, interval used in legend (e.g. 0, 1, 0.1), 4.  legend bar text (e.g. 'score (-)')
+    raster::metadata(mod.mean) <- list(paste(variable.name, ref.id, "mod_mean", sep = "_"), paste("Mean", meta.data.mod, sep = "_"),
+        mmi.mean, variable.unit)
+    raster::metadata(ref.mean) <- list(paste(variable.name, ref.id, "ref_mean", sep = "_"), paste("Mean", meta.data.ref, sep = "_"),
+        mmi.mean, variable.unit)
+    raster::metadata(bias) <- list(paste(variable.name, ref.id, "bias", sep = "_"), paste("Bias", meta.data.com, sep = "_"),
         mmi.bias, variable.unit)
-    
+    raster::metadata(bias.score) <- list(paste(variable.name, ref.id, "bias_score", sep = "_"), paste("Bias_score", meta.data.com,
+        sep = "_"), mmi.bias.score, "score (-)")
     # create a netcdf file from stat.metric
-    
-    stat.metric <- raster::stack(mod.mean, ref.mean, bias)
+
+    stat.metric <- raster::stack(mod.mean, ref.mean, bias, bias.score)
     for (i in 1:raster::nlayers(stat.metric)) {
         data <- raster::subset(stat.metric, i:i)
         my.filename <- unlist(raster::metadata(data)[1])
@@ -419,7 +442,7 @@ scores.grid.notime <- function(long.name, nc.mod, nc.ref, mod.id, ref.id, unit.c
         my.filename <- gsub(".", "-", my.filename, fixed = TRUE)
         my.longname <- unlist(raster::metadata(data)[2])
         if (outputDir != FALSE) {
-            raster::writeRaster(data, filename = paste(outputDir, my.filename, sep = "/"), format = "CDF", varname = variable.name, 
+            raster::writeRaster(data, filename = paste(outputDir, my.filename, sep = "/"), format = "CDF", varname = variable.name,
                 longname = my.longname, varunit = variable.unit, overwrite = TRUE)
         }
     }
